#package RPM::Source::Transformation::Resource::Repocop;
package Test::Repocop::RepocopResource;

use strict;
use warnings;
use Carp;
use Cwd;
use File::Basename;
use Test::Repocop::Workdir;
use Test::Repocop::Common;
use Test::Repocop::CLI::Container;
use Test::Repocop::CLI::Workdir;
use Test::Repocop::CLI::Report;
use Test::Repocop::TestDB;
use Test::Repocop::Metadata;
use RPM::Source::Transformation::Resource;
our @ISA=qw/RPM::Source::Transformation::Resource/;
push @RPM::Source::TransformContainer::RESOURCE_CLASSES, __PACKAGE__;

our $verbose=0;
our $opt_bypass_mode;
my $SRC2TEST={};
my $SRC2NAME={};
my ($metadata,$sourceid2specfilename);
my %SRCNAME2SOURCEID;

sub cli_container_ref {
    # not to drag short options
    #return ['Test::Repocop::CLI'];
    return ['Test::Repocop::CLI::Container'];
}

# hack
my $alttoolsdir=dirname($0);
$alttoolsdir=getcwd() if $alttoolsdir !~ m'^/home/';

# for cross-references in testnames
my @fixscriptdirs = ('/usr/share/repocop/fixscripts');
push @fixscriptdirs, $alttoolsdir.'/fixscripts' if -d $alttoolsdir.'/fixscripts';
push @INC, @fixscriptdirs;

my @fixscripts;
our @postcleanscripts;
foreach my $dir (@fixscriptdirs) {
    my @dir_scripts=map {basename($_)} map {s!.pl$!!;$_} glob ("$dir/*.pl");
    push @fixscripts, grep {!/^postclean-/}  @dir_scripts;
    push @postcleanscripts, grep {/^postclean-/} @dir_scripts;
}

our %test_with_fixscript=map {$_=>1} @fixscripts;

if ($verbose>2) {
    print STDERR "RepocopResource: available fixscripts:", join(',',keys(%test_with_fixscript)),"\n";
    print STDERR "RepocopResource: available postcleanscripts:", join(',',@postcleanscripts),"\n";
}

sub process_options {
    # call container's options first, as it calculate repocop_workdir
    map {$_->cli_process_options()} @{__PACKAGE__->cli_container_ref()};

    &Test::Repocop::Workdir::die_if_nothing_to_report();
    print STDERR "DEBUG: RepocopResource:process_options\n" if $verbose>2;
    $metadata=Test::Repocop::Metadata->new();
    $sourceid2specfilename=Test::Repocop::Metadata::sourceid2specfilename();
    my $iterator3=Test::Repocop::TestDB->new()->get_pkg_test_status_iterator();

    while (my ($rpmid,$test,$status)=$iterator3->iterate3_filter_by_test_status()) {
	#print STDERR "DEBUG: RepocopResource:iterate3: $rpmid:$test:$status\n" if $verbose>3;
	die "RepocopResource: bad --repocop-report-level: $status" if $status eq 'skip' or $status eq 'ok';
	next if not $test_with_fixscript{$test};
	my $name=$metadata->name($rpmid);
	my $sourceid=$metadata->sourceid($rpmid);
	my $sourcename=$metadata->name($sourceid);
	push @{$SRC2TEST->{$sourceid}}, $test;
	push @{$SRC2NAME->{$sourceid}}, $name;
	$SRCNAME2SOURCEID{$sourcename}=$sourceid;
    }
}

sub prepare_resource {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    return if $opt_bypass_mode;
    my $sourceid=$opt->{-repocop_sourceid};
    unless ($sourceid) {
	my $name=$spec->main_section->get_tag('Name');
	my $sourceid=$SRCNAME2SOURCEID{$name};
	die "RepocopResource: can't find sourceid for $name\n" unless defined $sourceid;
    }
    my $tests_ref=$SRC2TEST->{$sourceid};
    return unless $tests_ref;
    my @tests=@$tests_ref;
    my @names=@{$SRC2NAME->{$sourceid}};
    $opt->{-repocop_testnames}=\@tests;
    $opt->{-repocop_pkgnames}=\@names;

    # diff options
    my $diffname=$sourceid;
    $diffname=~s/\.src$//;
    $opt->{-diff_name}=$diffname;
    $opt->{-diff_specname}=$sourceid2specfilename->{$sourceid};
    # end diff options
}

sub sourceid_has_fixscripts {
    my ($sourceid)=@_;
    return $SRC2TEST->{$sourceid};
}

## TODO: wrap as some kind of Transformation::RepocopIdInputHandler ?
sub process_arguments {
    &Test::Repocop::CLI::Report::process_arguments();
    foreach my $file (@Repocop::arg::pkglist) {
	my ($srpmfile, $specfile, $sourceid);
	if ($file=~/\.rpm$/) {
	    $srpmfile = $file;
	    $sourceid=&Test::Repocop::Common::rpmfile2key($srpmfile);
	} elsif ($file=~/\.spec$/) {
	    $specfile=$file;
	    $sourceid=&Test::Repocop::Common::specfile2srcid($specfile);
	} else {
	    warn "Invalid argument: $file: not a .rpm or .spec!\n";
	    next;
	}
	unless ($sourceid =~ /src$/) {
	    warn "process_arguments: $sourceid: not a src.rpm!\n";
	    next;
	}
	#next unless &Test::Repocop::RepocopResource::sourceid_has_fixscripts($sourceid);
	next unless &sourceid_has_fixscripts($sourceid);
	print "$sourceid:\n";
	eval {
	    #---------------------------------------------------------
	    my $opt_transform=RPM::Source::TransformContainer->mk_opt();
	    $opt_transform->{-repocop_sourceid}=$sourceid;
	    my $rpmeditor=RPM::Source::Editor->new(
		SOURCERPM => $srpmfile,
		SPECFILE=> $specfile, 
		VERBOSE=> $Test::Repocop::Log::verbose,
		);
	    RPM::Source::TransformContainer->transform_once($rpmeditor,undef,$opt_transform);
	    #---------------------------------------------------------
	};
	warn "Got exception: $@" if $@;
    }
}

package Source::Shared::ResourceContainer;

# nothing ...

1;

__END__
